package com.sky.activate.tool;

import javax.servlet.http.HttpServletRequest;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.FileOutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.UUID;

public class Tools {

    private static Tools tools = new Tools();
    private Tools(){};
    public static Tools getInstance(){
        if(tools == null)
            tools = new Tools();
        return tools;
    }


    private long currTime = System.currentTimeMillis();

    public long getCurrTime() {
        return currTime;
    }

    /**
     * 获取权限最大值
     * @param roleStr
     * @return
     */
    public int getRoleMax(String roleStr){
        String[] roles = roleStr.split(",");
        int max = 0;
        for(String role : roles){
            int rol = Integer.valueOf(role);
            if(max < rol){
                max = rol;
            }
        }
        return max;
    }

    /**
     * 获取ip地址
     * @param request
     * @return
     */
    public String getIPAddress(HttpServletRequest request) {
        String ip = null;
        //X-Forwarded-For：Squid 服务代理
        String ipAddresses = request.getHeader("X-Forwarded-For");https://item.jd.com/10041580245532.html#crumb-wrap
        if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
            //Proxy-Client-IP：apache 服务代理
            ipAddresses = request.getHeader("Proxy-Client-IP");
        }
        if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
            //WL-Proxy-Client-IP：weblogic 服务代理
            ipAddresses = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
            //HTTP_CLIENT_IP：有些代理服务器
            ipAddresses = request.getHeader("HTTP_CLIENT_IP");
        }
        if (ipAddresses == null || ipAddresses.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
            //X-Real-IP：nginx服务代理
            ipAddresses = request.getHeader("X-Real-IP");
        }
        //有些网络通过多层代理，那么获取到的ip就会有多个，一般都是通过逗号（,）分割开来，并且第一个ip为客户端的真实IP
        if (ipAddresses != null && ipAddresses.length() != 0) {
            ip = ipAddresses.split(",")[0];
        }
        //还是不能获取到，最后再通过request.getRemoteAddr();获取
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ipAddresses)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    /***
     * 生成随机的UUID + yyyy-MM-dd
     * @return
     */
    public String getUUIDTime(){
        String str = "yyy-MM-dd";
        SimpleDateFormat sdf = new SimpleDateFormat(str);
        return UUID.randomUUID().toString() + "-" + sdf.format(new java.util.Date());
    }

    /***
     * 生成随机的UUID
     * @return
     */
    public String getUUID(){
        return UUID.randomUUID().toString();
    }

    /**
     * @param request 请求对象
     * @param clazz   要设置Bean的类型,传入试为: Bean.class
     * @return T
     */
    @SuppressWarnings("unchecked")
    public <T> T getEntityByRequest(HttpServletRequest request, Class<T> clazz){

        //获取页面所有的请求参数名称
        Enumeration<String> enume = request.getParameterNames();
        T beanObj = getObjectByClass(clazz);
        try{
            while(enume.hasMoreElements()){
                //参数名称
                String propertyName = enume.nextElement();
                //判断是否存在此属性
                if(isCheckBeanExitsPropertyName(clazz,propertyName)){
                    //获取请求值
                    Object propertyValue = request.getParameter(propertyName);
                    setProperties(beanObj,propertyName,propertyValue);
                }
            }
        }catch(Exception e){
//            System.out.println(Tools.class.getPackageName() + ":" + e.toString());
        }
        return beanObj;
    }

    private <T> T getObjectByClass(Class<T> clazz){
        T t = null;
        try {
            t = clazz.newInstance();
        } catch (InstantiationException e1) {
            e1.printStackTrace();
        } catch (IllegalAccessException e1) {
            e1.printStackTrace();
        }
        return t;
    }

    /**
     * @param clazz           Class对象
     * @param propertyName    属性名称
     * @return true || false  检查对象中是否存在该属性名称
     */
    private boolean isCheckBeanExitsPropertyName(Class<?> clazz,String propertyName){
        boolean retValue = false;
        try {
            Field field =  clazz.getDeclaredField(propertyName);
            if(null != field){
                retValue = true;
            }
        } catch (NoSuchFieldException e) {
            System.out.println("类: " + clazz.getSimpleName()+",不存在属性名: "+propertyName+" ,详细错误信息: "+e.getMessage());
        }
        return retValue;

    }


    /**
     * 设置字段值
     * @param object       实例对象
     * @param propertyName 属性名
     * @param value        新的字段值
     * @return
     */
    public void setProperties(Object object, String propertyName,Object value) throws IntrospectionException,
            IllegalAccessException, InvocationTargetException {
        PropertyDescriptor pd = new PropertyDescriptor(propertyName,object.getClass());
        Method methodSet = pd.getWriteMethod();
        if(pd.getPropertyType().getName().equals(Boolean.class.getName())){
            methodSet.invoke(object,Boolean.valueOf(value + ""));
        } else if(pd.getPropertyType().getName().equals(Integer.class.getName())){
            methodSet.invoke(object,Integer.valueOf(value + ""));
        } else if(pd.getPropertyType().getName().equals(Double.class.getName())){
            methodSet.invoke(object,Double.valueOf(value + ""));
        } else if(pd.getPropertyType().getName().equals(Float.class.getName())){
            methodSet.invoke(object,Float.valueOf(value + ""));
        } else if(pd.getPropertyType().getName().equals(Date.class.getName())){
            methodSet.invoke(object,Date.valueOf(value + ""));
        } else {
            methodSet.invoke(object,value);
        }
    }

    /***
     * 判断纬度是否合法，
     * @param lats 纬度集，范围是：-85 —— 85
     * @return
     */
    public boolean isLegalLat(Double ... lats){
        for (Double lat:
             lats) {
            if(lat == null || lat > 85 || lat < -85 ){
                return false;
            }
        }
        return true;
    }

    /***
     * 判断经度是否合法，
     * @param lons 经度集，范围是：-180 —— 180
     * @return
     */
    public boolean isLegalLon(Double ... lons){
        for (Double lon:
                lons) {
            if(lon == null || lon > 180 || lon < -180 ){
                return false;
            }
        }
        return true;
    }

    /***
     * 文件流保存到本地
     * @param file
     * @param filePath
     * @param fileName
     * @throws Exception
     */
    public void uploadFile(byte[] file, String filePath, String fileName) throws Exception {
        File targetFile = new File(filePath);
        if(!targetFile.exists()){
            targetFile.mkdirs();
        }
        FileOutputStream out = new FileOutputStream(filePath + fileName);
        out.write(file);
        out.flush();
        out.close();
    }


    public static String str = "yyy-MM-dd HH:mm:ss";
    public static SimpleDateFormat sdf = new SimpleDateFormat(str);

    /***
     * 获取字符串类型时间
     * @param date
     * @return
     */
    public String getDateYYYYmmDDHHmmss(java.util.Date date){
        return sdf.format(date);
    }


}
